"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.make = exports.layerConfig = exports.layer = exports.TypeId = exports.SqliteClient = void 0;
var Reactivity = _interopRequireWildcard(require("@effect/experimental/Reactivity"));
var Client = _interopRequireWildcard(require("@effect/sql/SqlClient"));
var _SqlError = require("@effect/sql/SqlError");
var Statement = _interopRequireWildcard(require("@effect/sql/Statement"));
var Chunk = _interopRequireWildcard(require("effect/Chunk"));
var Config = _interopRequireWildcard(require("effect/Config"));
var Context = _interopRequireWildcard(require("effect/Context"));
var Effect = _interopRequireWildcard(require("effect/Effect"));
var _Function = require("effect/Function");
var Layer = _interopRequireWildcard(require("effect/Layer"));
var Scope = _interopRequireWildcard(require("effect/Scope"));
var Stream = _interopRequireWildcard(require("effect/Stream"));
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function (e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { default: e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n.default = e, t && t.set(e, n), n; }
/**
 * @category type ids
 * @since 1.0.0
 */
const TypeId = exports.TypeId = /*#__PURE__*/Symbol.for("@effect/sql-sqlite-do/SqliteClient");
/**
 * @category tags
 * @since 1.0.0
 */
const SqliteClient = exports.SqliteClient = /*#__PURE__*/Context.GenericTag("@effect/sql-sqlite-do/SqliteClient");
/**
 * @category constructor
 * @since 1.0.0
 */
const make = options => Effect.gen(function* () {
  const compiler = Statement.makeCompilerSqlite(options.transformQueryNames);
  const transformRows = options.transformResultNames ? Statement.defaultTransforms(options.transformResultNames).array : undefined;
  const makeConnection = Effect.gen(function* () {
    const db = options.db;
    function* runIterator(sql, params = []) {
      const cursor = db.exec(sql, ...params);
      const columns = cursor.columnNames;
      for (const result of cursor.raw()) {
        const obj = {};
        for (let i = 0; i < columns.length; i++) {
          const value = result[i];
          obj[columns[i]] = value instanceof ArrayBuffer ? new Uint8Array(value) : value;
        }
        yield obj;
      }
    }
    const runStatement = (sql, params = []) => Effect.try({
      try: () => Array.from(runIterator(sql, params)),
      catch: cause => new _SqlError.SqlError({
        cause,
        message: `Failed to execute statement`
      })
    });
    const runValues = (sql, params = []) => Effect.try({
      try: () => Array.from(db.exec(sql, ...params).raw(), row => {
        for (let i = 0; i < row.length; i++) {
          const value = row[i];
          if (value instanceof ArrayBuffer) {
            row[i] = new Uint8Array(value);
          }
        }
        return row;
      }),
      catch: cause => new _SqlError.SqlError({
        cause,
        message: `Failed to execute statement`
      })
    });
    return (0, _Function.identity)({
      execute(sql, params, transformRows) {
        return transformRows ? Effect.map(runStatement(sql, params), transformRows) : runStatement(sql, params);
      },
      executeRaw(sql, params) {
        return runStatement(sql, params);
      },
      executeValues(sql, params) {
        return runValues(sql, params);
      },
      executeUnprepared(sql, params, transformRows) {
        return transformRows ? Effect.map(runStatement(sql, params), transformRows) : runStatement(sql, params);
      },
      executeStream(sql, params, transformRows) {
        return Stream.suspend(() => {
          const iterator = runIterator(sql, params);
          return Stream.fromIteratorSucceed(iterator, 16);
        }).pipe(transformRows ? Stream.mapChunks(chunk => Chunk.unsafeFromArray(transformRows(Chunk.toReadonlyArray(chunk)))) : _Function.identity);
      }
    });
  });
  const semaphore = yield* Effect.makeSemaphore(1);
  const connection = yield* makeConnection;
  const acquirer = semaphore.withPermits(1)(Effect.succeed(connection));
  const transactionAcquirer = Effect.uninterruptibleMask(restore => Effect.as(Effect.zipRight(restore(semaphore.take(1)), Effect.tap(Effect.scope, scope => Scope.addFinalizer(scope, semaphore.release(1)))), connection));
  return Object.assign(yield* Client.make({
    acquirer,
    compiler,
    transactionAcquirer,
    spanAttributes: [...(options.spanAttributes ? Object.entries(options.spanAttributes) : []), ["db.system", "sqlite"]],
    transformRows
  }), {
    [TypeId]: TypeId,
    config: options
  });
});
/**
 * @category layers
 * @since 1.0.0
 */
exports.make = make;
const layerConfig = config => Layer.scopedContext(Config.unwrap(config).pipe(Effect.flatMap(make), Effect.map(client => Context.make(SqliteClient, client).pipe(Context.add(Client.SqlClient, client))))).pipe(Layer.provide(Reactivity.layer));
/**
 * @category layers
 * @since 1.0.0
 */
exports.layerConfig = layerConfig;
const layer = config => Layer.scopedContext(Effect.map(make(config), client => Context.make(SqliteClient, client).pipe(Context.add(Client.SqlClient, client)))).pipe(Layer.provide(Reactivity.layer));
exports.layer = layer;
//# sourceMappingURL=SqliteClient.js.map